import uuid

from django.conf import settings
from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import PermissionsMixin
from django.db import models
from django_prometheus.models import ExportModelOperationsMixin
from django_scopes import ScopedManager

from account.managers import UserManager


class Organization(ExportModelOperationsMixin('company'), models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=255, unique=True)
    slug = models.SlugField(unique=True, null=True)
    kvk_number = models.CharField(max_length=20, blank=True, null=True)
    vat_number = models.CharField(max_length=14, blank=True, null=True)
    iban = models.CharField(max_length=34, blank=True, null=True)
    address = models.CharField(max_length=255, blank=True, null=True)
    postal_code = models.CharField(max_length=20, blank=True, null=True)
    city = models.CharField(max_length=100, blank=True, null=True)
    email = models.EmailField(blank=True, null=True)

    users = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        through='Membership',
        related_name='organizations'
    )
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    objects = models.Manager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = "Organizations"
        ordering = ['name']

class User(ExportModelOperationsMixin('user'), AbstractBaseUser, PermissionsMixin):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    email = models.EmailField(unique=True)

    is_staff = models.BooleanField(default=False)
    is_active = models.BooleanField(default=True)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    objects = UserManager()

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    groups = models.ManyToManyField(
        'auth.Group',
        verbose_name='groups',
        blank=True,
        help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.',
        related_name="account_user_groups",
    )
    user_permissions = models.ManyToManyField(
        'auth.Permission',
        verbose_name='user permissions',
        blank=True,
        help_text='Specific permissions for this user.',
        related_name="account_user_permissions",
    )

    @property
    def has_multiple_organizations(self):
        """Geeft True terug als de gebruiker bij meer dan 1 organisatie hoort."""
        return self.memberships.count() > 1

    def is_owner_of(self, organization):
        return self.memberships.filter(organization=organization, role='owner').exists()

    def is_admin_of(self, organization):
        return self.memberships.filter(organization=organization, role='admin').exists()

    def __str__(self):
        return self.email

    class Meta:
        verbose_name = 'user'
        verbose_name_plural = 'users'


class UserProfile(models.Model):
    # De koppeling naar de User
    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='profile')

    # De verplaatste velden
    first_name = models.CharField(max_length=30, blank=True)
    last_name = models.CharField(max_length=30, blank=True)

    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return f"Profiel van {self.user.email}"

class Membership(models.Model):
    ROLES = (
        ('owner', 'Eigenaar'),
        ('admin', 'Beheerder'),  # Kan instellingen wijzigen
        ('member', 'Medewerker'),  # Kan facturen maken
        ('viewer', 'Boekhouder'),  # Kan alleen kijken
    )

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='memberships')
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE)
    role = models.CharField(max_length=20, choices=ROLES, default='member')
    joined_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        unique_together = ('user', 'organization') # Prevent double memberships
        ordering = ['joined_at']


class TenantAwareModel(models.Model):
    """
    Abstract model to make sure that data ALWAYS belongs to an organization.
    Use this for EVERYTHING in your SaaS system (Invoices, Clients, Tasks, etc).
    """
    organization = models.ForeignKey(
        Organization,
        on_delete=models.CASCADE,
        related_name="%(class)s_set"  # Gives for example: organization.invoice_set
    )

    # This makes sure that Invoice.objects.all() automatically filters on the active organization.
    objects = ScopedManager(organization='organization')

    class Meta:
        abstract = True



import unittest

from account.forms import OrganizationForm


class CleanKvkNumberTests(unittest.TestCase):
    # pylint: disable=invalid-name
    def test_clean_kvk_number_happy_paths(self):
        # Arrange
        cases = [
            {
                "id": "happy-all-different-digits",
                "kvk_input": "12345678",
                "expected_output": "12345678",
            },
            {
                "id": "happy-all-zeros",
                "kvk_input": "00000000",
                "expected_output": "00000000",
            },
            {
                "id": "happy-random-valid",
                "kvk_input": "98765432",
                "expected_output": "98765432",
            },
        ]

        for case in cases:
            with self.subTest(case_id=case["id"]):

                # Arrange
                data = {
                    "name": f"Test Org {case['id']}",
                    "slug": f"test-org-{case['id']}",
                    "kvk_number": case["kvk_input"]
                }
                form = OrganizationForm(data=data)

                # Act
                is_valid = form.is_valid()
                kvk_cleaned = form.cleaned_data.get("kvk_number") if is_valid else None

                # Assert
                self.assertTrue(is_valid, f"Form errors: {form.errors}")
                self.assertEqual(kvk_cleaned, case["expected_output"])

    # pylint: disable=invalid-name
    def test_clean_kvk_number_edge_cases(self):
        # Arrange
        cases = [
            {
                "id": "edge-none-value",
                "data": {"kvk_number": None},
                "expected_output": None,
            },
            {
                "id": "edge-empty-string",
                "data": {"kvk_number": ""},
                "expected_output": None,
            },
        ]

        for case in cases:
            with self.subTest(case_id=case["id"]):

                # Arrange
                form = OrganizationForm(data=case["data"])
                print(case['id'])
                # Act
                is_valid = form.is_valid()
                kvk_cleaned = form.cleaned_data.get("kvk_number") if is_valid else None

                # Assert
                self.assertFalse(is_valid)
                self.assertEqual(kvk_cleaned, case["expected_output"])

    # pylint: disable=invalid-name
    def test_clean_kvk_number_error_cases(self):
        # Arrange
        cases = [
            {
                "id": "error-too-short-7-digits",
                "kvk_input": "1234567",
            },
            {
                "id": "error-too-long-9-digits",
                "kvk_input": "123456789",
            },
            {
                "id": "error-letters-only",
                "kvk_input": "ABCDEFGH",
            },
            {
                "id": "error-mixed-alnum",
                "kvk_input": "1234ABCD",
            },
            {
                "id": "error-space-in-string",
                "kvk_input": "1234 678",
            },
            {
                "id": "error-hyphen-in-string",
                "kvk_input": "1234-678",
            },
            {
                "id": "error-non-ascii-digits",
                "kvk_input": "１２３４５６７８",
            },
        ]
        expected_error = "Een KvK-nummer moet precies 8 cijfers bevatten."

        for case in cases:
            with self.subTest(case_id=case["id"]):

                # Arrange
                form = OrganizationForm(data={"kvk_number": case["kvk_input"]})

                # Act
                is_valid = form.is_valid()
                errors = form.errors.get("kvk_number", [])

                # Assert
                self.assertFalse(is_valid)
                self.assertIn(expected_error, errors)


if __name__ == "__main__":
    unittest.main()



import unittest

from django.contrib.auth import get_user_model
from django.test import TestCase, Client, RequestFactory
from django.urls import reverse

from account.forms import LoginForm
from account.views import CustomLoginView

User = get_user_model()


class CustomLoginViewAuthenticatedUserTests(TestCase):
    def setUp(self):
        # Arrange
        self.client = Client()

    # pylint: disable=invalid-name
    def test_authenticated_user_get_redirects_various(self):
        # Arrange
        cases = [
            {
                "id": "happy-authenticated-redirect-1",
                "email": "user1@example.com",
                "password": "StrongPass123",
            },
            {
                "id": "happy-authenticated-redirect-2",
                "email": "user2@example.com",
                "password": "AnotherStrong456",
            },
            {
                "id": "happy-authenticated-redirect-3",
                "email": "minimal@example.com",
                "password": "PwdLen8!!",
            },
        ]
        login_url = reverse("login")

        for case in cases:
            with self.subTest(case_id=case["id"]):
                # Arrange
                User.objects.create_user(
                    email=case["email"],
                    password=case["password"],
                )
                logged_in = self.client.login(
                    email=case["email"],
                    password=case["password"],
                )
                self.assertTrue(logged_in)

                # Act
                response = self.client.get(login_url)

                # Assert
                self.assertEqual(response.status_code, 302)
                self.assertNotEqual(response.url, login_url)


class CustomLoginViewAnonymousGetTests(TestCase):
    def setUp(self):
        # Arrange
        self.client = Client()

    # pylint: disable=invalid-name
    def test_anonymous_get_sees_login_page_various(self):
        # Arrange
        cases = [
            {
                "id": "edge-anonymous-get-login-1",
                "email": "anon1@example.com",
                "password": "SomePassword!",
            },
            {
                "id": "edge-anonymous-get-login-2",
                "email": "anon2@example.com",
                "password": "AnotherPwd123",
            },
        ]
        login_url = reverse("login")

        for case in cases:
            with self.subTest(case_id=case["id"]):
                # Act
                response = self.client.get(login_url)

                # Assert
                self.assertEqual(response.status_code, 200)
                template_names = [t.name for t in response.templates if t.name]
                self.assertIn("account/login.html", template_names)
                self.assertIn("form", response.context)
                self.assertIsInstance(response.context["form"], LoginForm)


class CustomLoginViewPostAuthenticationTests(TestCase):
    def setUp(self):
        # Arrange
        self.client = Client()
        self.actual_email = "validuser@example.com"
        self.actual_password = "ValidPass123!"
        User.objects.create_user(
            email=self.actual_email,
            password=self.actual_password,
        )

    # pylint: disable=invalid-name
    def test_post_authentication_various_cases(self):
        # Arrange
        cases = [
            {
                "id": "happy-valid-credentials",
                "email": "validuser@example.com",
                "password": "ValidPass123!",
                "expected_valid": True,
            },
            {
                "id": "error-invalid-password",
                "email": "validuser@example.com",
                "password": "WrongPassword",
                "expected_valid": False,
            },
            {
                "id": "error-nonexistent-user",
                "email": "nonexistent@example.com",
                "password": "Whatever123",
                "expected_valid": False,
            },
        ]
        login_url = reverse("login")

        for case in cases:
            with self.subTest(case_id=case["id"]):
                # Belangrijk: Maak de client leeg voor elke subtest om sessie-vervuiling te voorkomen
                self.client.logout()

                login_data = {
                    "username": case["email"],
                    "password": case["password"],
                }

                # Gebruik follow=False om de crash op het dashboard te voorkomen
                response = self.client.post(login_url, data=login_data, follow=False)

                if case["expected_valid"]:
                    # Verwacht een 302 naar de succes-pagina
                    self.assertEqual(response.status_code, 302)
                    self.assertIn(response.url, ['/', reverse('dashboard')])  # Pas aan naar jouw succes-URL
                else:
                    # Als dit nu NOG STEEDS een 302 geeft, redirect je view
                    # de gebruiker ook bij foute credentials.
                    self.assertEqual(response.status_code, 200,
                                     msg=f"Fout bij {case['id']}: Verwachtte 200 (form error), kreeg {response.status_code} naar {response.get('Location')}")

                    self.assertTemplateUsed(response, "account/login.html")
                    self.assertFalse(response.context["form"].is_valid())

class CustomLoginViewInvalidInputEdgeCasesTests(TestCase):
    def setUp(self):
        # Arrange
        self.client = Client()

    # pylint: disable=invalid-name
    def test_post_invalid_input_edge_cases(self):
        # Arrange
        cases = [
            {
                "id": "error-empty-username-and-password",
                "username": "",
                "password": "",
            },
            {
                "id": "error-whitespace-username",
                "username": "     ",
                "password": "somepassword",
            },
            {
                "id": "error-empty-password",
                "username": "user@example.com",
                "password": "",
            },
        ]
        login_url = reverse("login")

        for case in cases:
            with self.subTest(case_id=case["id"]):
                # Act
                response = self.client.post(
                    login_url,
                    data={"username": case["username"], "password": case["password"]},
                )

                # Assert
                self.assertEqual(response.status_code, 200)
                template_names = [t.name for t in response.templates if t.name]
                self.assertIn("account/login.html", template_names)
                form = response.context["form"]
                self.assertIsInstance(form, LoginForm)
                self.assertFalse(form.is_valid())


class CustomLoginViewConfigurationTests(TestCase):
    # pylint: disable=invalid-name
    def test_custom_login_view_configuration(self):
        # Arrange
        view = CustomLoginView()

        # Act
        template_name = view.template_name
        authentication_form = view.authentication_form
        redirect_authenticated_user = view.redirect_authenticated_user

        # Assert
        self.assertEqual(template_name, "account/login.html")
        self.assertIs(authentication_form, LoginForm)
        self.assertTrue(redirect_authenticated_user)


class CustomLoginViewRequestFactoryTests(TestCase):
    def setUp(self):
        # Arrange
        self.factory = RequestFactory()

if __name__ == "__main__":
    unittest.main()



from django.db.models.signals import post_save
from django.dispatch import receiver
from .models import User, UserProfile

@receiver(post_save, sender=User)
def create_user_profile(sender, instance, created, **kwargs):
    if created:
        UserProfile.objects.create(user=instance)

@receiver(post_save, sender=User)
def save_user_profile(sender, instance, **kwargs):
    instance.profile.save()



import factory
from django.utils.text import slugify
from django.contrib.auth import get_user_model
from faker import Faker

from account.models import Organization, Membership, UserProfile

User = get_user_model()
fake = Faker('nl_NL')

class UserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = User
        django_get_or_create = ('email',)

    email = factory.Faker('email', locale='nl_NL')
    password = factory.PostGenerationMethodCall('set_password', 'password123')

    @factory.post_generation
    def first_name(obj, create, extracted, **kwargs):
        if not create:
            return

        # De signal heeft het profiel al gemaakt, we passen het alleen aan
        profile, _ = UserProfile.objects.get_or_create(user=obj)
        profile.first_name = extracted or fake.first_name()
        profile.save()

    @factory.post_generation
    def last_name(obj, create, extracted, **kwargs):
        if not create:
            return

        profile, _ = UserProfile.objects.get_or_create(user=obj)
        profile.last_name = extracted or fake.last_name()
        profile.save()

class UserProfileFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = UserProfile
        # DIT IS DE FIX: Het voorkomt dat hij een nieuw profiel probeert te maken
        # als de signal er al een heeft aangemaakt voor deze user.
        django_get_or_create = ('user',)

    user = factory.SubFactory(UserFactory)
    first_name = factory.Faker('first_name', locale='nl_NL')
    last_name = factory.Faker('last_name', locale='nl_NL')


class OrganizationFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Organization

    name = factory.Faker('company', locale='nl_NL')
    slug = factory.LazyAttribute(lambda o: slugify(o.name))


class MembershipFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Membership

    user = factory.SubFactory(UserFactory)
    organization = factory.SubFactory(OrganizationFactory)
    role = 'owner'



# Generated by Django 6.0.2 on 2026-02-12 14:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='organization',
            name='address',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='organization',
            name='city',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='organization',
            name='email',
            field=models.EmailField(blank=True, max_length=254, null=True),
        ),
        migrations.AddField(
            model_name='organization',
            name='postal_code',
            field=models.CharField(blank=True, max_length=20, null=True),
        ),
    ]



from django.urls import path
from django.contrib.auth import views as auth_views
from .views import CustomLoginView, RegisterView, UserProfileView, CompanySettingsView, SelectOrganizationView

urlpatterns = [
    path('register/', RegisterView.as_view(), name='register'),
    path('login/', CustomLoginView.as_view(), name='login'),
    path('logout/', auth_views.LogoutView.as_view(), name='logout'),
    path('select-organization/', SelectOrganizationView.as_view(), name='select_organization'),
    path('profile/', UserProfileView.as_view(), name='user_profile'),
    path('settings/organization/', CompanySettingsView.as_view(), name='company_settings'),
]



import re

from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import UserCreationForm, AuthenticationForm

from account.models import Organization, UserProfile

User = get_user_model()

class RegistrationForm(UserCreationForm):
    company_name = forms.CharField(
        label="Bedrijfsnaam",
        max_length=255,
        widget=forms.TextInput(attrs={
            'placeholder': 'Mijn Bouwbedrijf B.V.',
        })
    )

    email = forms.EmailField(
        required=True,
        widget=forms.EmailInput(attrs={
            'placeholder': 'naam@bedrijf.nl'
        })
    )

    class Meta:
        model = User
        fields = ('email',)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        for _field_name, field in self.fields.items():
            field.widget.attrs.update({
                'class': 'block w-full appearance-none rounded-md border border-gray-300 px-3 py-3 placeholder-gray-400 shadow-sm focus:border-blue-500 focus:outline-none focus:ring-blue-500 sm:text-sm'
            })

        self.fields['password1'].help_text = "Gebruik minimaal 8 tekens."


class LoginForm(AuthenticationForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields['username'].label = "E-mailadres"
        self.fields['username'].widget.attrs.update({
            'placeholder': 'naam@bedrijf.nl',
            'class': 'block w-full appearance-none rounded-md border border-gray-300 px-3 py-3 placeholder-gray-400 shadow-sm focus:border-blue-500 focus:outline-none focus:ring-blue-500 sm:text-sm'
        })

        self.fields['password'].label = "Wachtwoord"
        self.fields['password'].widget.attrs.update({
            'class': 'block w-full appearance-none rounded-md border border-gray-300 px-3 py-3 placeholder-gray-400 shadow-sm focus:border-blue-500 focus:outline-none focus:ring-blue-500 sm:text-sm'
        })

class OrganizationForm(forms.ModelForm):
    class Meta:
        model = Organization
        fields = ['name', 'slug', 'address', 'postal_code', 'city', 'email', 'kvk_number', 'vat_number', 'iban']
        widgets = {
            field: forms.TextInput(attrs={
                'class': 'mt-1 block w-full border border-gray-300 rounded-md shadow-sm py-2 px-3 focus:outline-none focus:ring-blue-500 focus:border-blue-500 sm:text-sm'
            }) for field in ['name', 'slug', 'kvk_number', 'vat_number', 'iban']
        }

    def clean_postal_code(self):
        postal_code = self.cleaned_data.get('postal_code')
        if postal_code:
            postal_code = postal_code.replace(" ", "").upper()
            # Nederlandse postcode check: 4 cijfers, 2 letters
            if not re.match(r'^[1-9][0-9]{3}[A-Z]{2}$', postal_code):
                raise forms.ValidationError("Voer een geldige Nederlandse postcode in (bijv. 9648JX).")
            # We slaan hem netjes geformatteerd op: 1234 AB
            return f"{postal_code[:4]} {postal_code[4:]}"
        return postal_code

    def clean_kvk_number(self):
        kvk = self.cleaned_data.get('kvk_number')
        if kvk:
            # Use [0-9] to strictly enforce ASCII digits. \d matches unicode digits too.
            if not re.match(r'^[0-9]{8}$', kvk):
                raise forms.ValidationError("Een KvK-nummer moet precies 8 cijfers bevatten.")
        return kvk

    def clean_vat_number(self):
        vat = self.cleaned_data.get('vat_number')
        if vat:
            # Format: NL + 9 cijfers + B + 2 cijfers
            pattern = r'^NL\d{9}B\d{2}$'
            if not re.match(pattern, vat):
                raise forms.ValidationError("Voer een geldig BTW-nummer in (bijv. NL123456789B01).")
        return vat

    def clean_iban(self):
        iban = self.cleaned_data.get('iban')
        if iban:
            iban = iban.replace(" ", "").upper()
            if not re.match(r'^[A-Z]{2}\d{2}[A-Z]{4}\d{10}$', iban):
                raise forms.ValidationError("Voer een geldig IBAN nummer in.")
        return iban

class UserProfileForm(forms.ModelForm):
    class Meta:
        model = UserProfile
        fields = ['first_name', 'last_name']
        labels = {
            'first_name': 'Voornaam',
            'last_name': 'Achternaam',
        }
        widgets = {
            'first_name': forms.TextInput(attrs={
                'class': 'mt-1 block w-full border border-gray-300 rounded-md shadow-sm py-2 px-3 focus:outline-none focus:ring-blue-500 focus:border-blue-500 sm:text-sm'
            }),
            'last_name': forms.TextInput(attrs={
                'class': 'mt-1 block w-full border border-gray-300 rounded-md shadow-sm py-2 px-3 focus:outline-none focus:ring-blue-500 focus:border-blue-500 sm:text-sm'
            }),
        }



# Generated by Django 6.0.2 on 2026-02-10 08:45

import django.db.models.deletion
import django_prometheus.models
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '__first__'),
        ('auth', '__first__'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('email', models.EmailField(max_length=254, unique=True)),
                ('is_staff', models.BooleanField(default=False)),
                ('is_active', models.BooleanField(default=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='account_user_groups', to='auth.group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='account_user_permissions', to='auth.permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
            },
            bases=(django_prometheus.models.ExportModelOperationsMixin('user'), models.Model),
        ),
        migrations.CreateModel(
            name='Membership',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('role', models.CharField(choices=[('owner', 'Eigenaar'), ('admin', 'Beheerder'), ('member', 'Medewerker'), ('viewer', 'Boekhouder')], default='member', max_length=20)),
                ('joined_at', models.DateTimeField(auto_now_add=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='memberships', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['joined_at'],
            },
        ),
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255, unique=True)),
                ('slug', models.SlugField(null=True, unique=True)),
                ('kvk_number', models.CharField(blank=True, max_length=20, null=True)),
                ('vat_number', models.CharField(blank=True, max_length=14, null=True)),
                ('iban', models.CharField(blank=True, max_length=34, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('users', models.ManyToManyField(related_name='organizations', through='account.Membership', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'Organizations',
                'ordering': ['name'],
            },
            bases=(django_prometheus.models.ExportModelOperationsMixin('company'), models.Model),
        ),
        migrations.AddField(
            model_name='membership',
            name='organization',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='account.organization'),
        ),
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(blank=True, max_length=30)),
                ('last_name', models.CharField(blank=True, max_length=30)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='profile', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='membership',
            unique_together={('user', 'organization')},
        ),
    ]



from django import template

register = template.Library()


@register.filter
def is_owner(user, organization):
    """
    Checkt of een gebruiker de 'owner' rol heeft voor een specifieke organisatie.
    Gebruik: {% if user|is_owner:request.org %}
    """
    if not user or user.is_anonymous or not organization:
        return False

    # Dit roept de methode aan die in je User model staat (uit je diff)
    if hasattr(user, 'is_owner_of'):
        return user.is_owner_of(organization)
    return False

@register.filter
def is_admin(user, organization):
    """
    Checkt of een gebruiker de 'owner' rol heeft voor een specifieke organisatie.
    Gebruik: {% if user|is_owner:request.org %}
    """
    if not user or user.is_anonymous or not organization:
        return False

    # Dit roept de methode aan die in je User model staat (uit je diff)
    if hasattr(user, 'is_owner_of'):
        return user.is_owner_of(organization)
    if hasattr(user, 'is_admin_of'):
        return user.is_admin_of(organization)
    return False



from django_scopes import scope
from django.shortcuts import redirect
from django.urls import reverse
from account.models import Organization

class TenantMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if not request.user.is_authenticated:
            return self.get_response(request)

        active_org_id = request.session.get('active_org_id') # Laten we 'org' aanhouden ipv 'company'

        select_url = reverse('select_organization')
        logout_url = reverse('logout')
        exempt_urls = [select_url, logout_url]

        if not active_org_id and request.path not in exempt_urls:
            if first_org := request.user.organizations.first():
                active_org_id = str(first_org.id)
                request.session['active_org_id'] = active_org_id
            else:
                # @TODO: User doesn't have any organization -> Edge case!
                pass

        if active_org_id:
            try:
                org = Organization.objects.get(id=active_org_id, users=request.user)
                request.organization = org

                with scope(organization=org):
                    return self.get_response(request)

            except Organization.DoesNotExist:
                if 'active_org_id' in request.session:
                    del request.session['active_org_id']
                return redirect(reverse('dashboard'))

        return self.get_response(request)



from django.contrib.auth.mixins import UserPassesTestMixin
from django.core.exceptions import PermissionDenied


class TenantOwnerRequiredMixin(UserPassesTestMixin):
    """
    Mixin die controleert of de gebruiker de 'owner' rol heeft
    binnen de op dat moment actieve organisatie (request.org).
    """

    def test_func(self):
        active_org = getattr(self.request, 'organization', None)
        if not active_org:
            return False

        return self.request.user.is_authenticated and self.request.user.is_owner_of(active_org)

    def handle_no_permission(self):
        if not self.request.user.is_authenticated:
            return super().handle_no_permission()

        raise PermissionDenied("Je hebt geen eigenaarsrechten voor deze organisatie.")



from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.views import LoginView
from django.contrib.messages.views import SuccessMessageMixin
from django.shortcuts import redirect, render
from django.urls import reverse_lazy
from django.views import View
from django.views.generic import UpdateView
from django.views.generic.edit import CreateView
from django.contrib.auth import login
from .forms import RegistrationForm, LoginForm, OrganizationForm, UserProfileForm
from .mixins import TenantOwnerRequiredMixin
from .models import Organization, Membership, UserProfile


class RegisterView(CreateView):
    template_name = 'register/register.html'
    form_class = RegistrationForm
    success_url = reverse_lazy('login')

    def form_valid(self, form):
        user = form.save()
        company_name = form.cleaned_data.get('company_name')
        org = Organization.objects.create(name=company_name)
        Membership.objects.create(
            user=user,
            organization=org,
            role='owner'
        )
        login(self.request, user)

        return redirect(self.success_url)

class CustomLoginView(LoginView):
    template_name = 'account/login.html'
    authentication_form = LoginForm
    redirect_authenticated_user = True # Als ze al ingelogd zijn, stuur ze door naar core

class SelectOrganizationView(LoginRequiredMixin, View):
    def get(self, request):
        memberships = request.user.memberships.select_related('organization').all()

        if memberships.count() == 1:
            request.session['active_org_id'] = str(memberships.first().organization.id)
            return redirect('dashboard')

        return render(request, 'account/select_organization.html', {
            'memberships': memberships
        })

    def post(self, request):
        org_id = request.POST.get('org_id')
        request.session['active_org_id'] = org_id
        return redirect('dashboard')

class UserProfileView(LoginRequiredMixin, SuccessMessageMixin, UpdateView):
    model = UserProfile
    form_class = UserProfileForm
    template_name = 'account/profile.html'
    success_url = reverse_lazy('user_profile')
    success_message = "Je persoonlijke gegevens zijn bijgewerkt!"

    def get_object(self, queryset=None):
        return self.request.user.profile

class CompanySettingsView(TenantOwnerRequiredMixin, LoginRequiredMixin, SuccessMessageMixin, UpdateView):
    model = Organization
    form_class = OrganizationForm  # Gebruik nu het formulier
    template_name = 'account/company_settings.html'
    success_url = reverse_lazy('company_settings')
    success_message = "Bedrijfsnaam is succesvol aangepast!"

    def get_object(self, queryset=None):
        return self.request.organization



from django.apps import AppConfig

class AccountConfig(AppConfig):
    name = 'account'

    def ready(self):
        # pylint: disable=import-outside-toplevel
        from account import signals
        _ = signals



import unittest

from django import forms
from django.contrib.auth import get_user_model
from django.test import TestCase

from account.forms import RegistrationForm

User = get_user_model()


class TestRegistrationFormValidation(TestCase):
    # pylint: disable=invalid-name
    def test_registration_form_validation_various_cases(self):
        # Arrange
        cases = [
            # Happy paths
            {
                "id": "happy-basic-valid",
                "data": {
                    "email": "user@example.com",
                    "company_name": "Mijn Bouwbedrijf B.V.",
                    "password1": "strongPass123",
                    "password2": "strongPass123",
                },
                "expected_valid": True,
            },
            {
                "id": "happy-another-valid",
                "data": {
                    "email": "another.user@bedrijf.nl",
                    "company_name": "Ashlar Systems",
                    "password1": "anotherStrongPass456",
                    "password2": "anotherStrongPass456",
                },
                "expected_valid": True,
            },
            {
                "id": "happy-min-companyname-length",
                "data": {
                    "email": "short@co.nl",
                    "company_name": "A",
                    "password1": "passwordLen8",
                    "password2": "passwordLen8",
                },
                "expected_valid": True,
            },
            # Error: password mismatch
            {
                "id": "error-password-mismatch",
                "data": {
                    "email": "mismatch@bedrijf.nl",
                    "company_name": "Mismatch Company",
                    "password1": "password123",
                    "password2": "different123",
                },
                "expected_valid": False,
            },
            # Error: missing company_name
            {
                "id": "error-missing-company-name",
                "data": {
                    "email": "no-company@bedrijf.nl",
                    "company_name": "",
                    "password1": "password123",
                    "password2": "password123",
                },
                "expected_valid": False,
            },
            # Error: invalid email
            {
                "id": "error-invalid-email-format",
                "data": {
                    "email": "not-an-email",
                    "company_name": "Bad Email Company",
                    "password1": "password123",
                    "password2": "password123",
                },
                "expected_valid": False,
            },
        ]

        # Act
        for case in cases:
            with self.subTest(case_id=case["id"]):
                form = RegistrationForm(data=case["data"])

                # Assert
                self.assertIs(form.is_valid(), case["expected_valid"])
                if case["expected_valid"]:
                    self.assertNotIn("email", form.errors)
                    self.assertNotIn("company_name", form.errors)
                    self.assertNotIn("password1", form.errors)
                    self.assertNotIn("password2", form.errors)
                else:
                    self.assertTrue(form.errors)


class TestRegistrationFormMetaAndFields(TestCase):
    # pylint: disable=invalid-name
    def test_registration_form_meta_uses_user_and_email_field(self):
        # Arrange
        form = RegistrationForm()

        # Act
        meta_model = form._meta.model
        meta_fields = form._meta.fields

        # Assert
        self.assertIs(meta_model, User)
        self.assertEqual(meta_fields, ("email",))

    # pylint: disable=invalid-name
    def test_registration_form_applies_tailwind_classes_to_all_fields(self):
        # Arrange
        form = RegistrationForm()

        # Act
        widget_classes = {
            name: field.widget.attrs.get("class", "")
            for name, field in form.fields.items()
        }

        # Assert
        tailwind_snippet = "block w-full appearance-none rounded-md border"
        self.assertTrue(widget_classes, "Form should have fields")
        for field_name, classes in widget_classes.items():
            with self.subTest(field=field_name):
                self.assertIn(tailwind_snippet, classes)


    # pylint: disable=invalid-name
    def test_registration_form_sets_password1_help_text(self):
        # Arrange
        form = RegistrationForm()

        # Act
        help_text = form.fields["password1"].help_text

        # Assert
        self.assertEqual(help_text, "Gebruik minimaal 8 tekens.")

    # pylint: disable=invalid-name
    def test_registration_form_required_fields(self):
        # Arrange
        base_data = {
            "email": "required@bedrijf.nl",
            "company_name": "Required Co",
            "password1": "password123",
            "password2": "password123",
        }
        cases = [
            {
                "id": "missing-email",
                "missing_field": "email",
            },
            {
                "id": "missing-company-name",
                "missing_field": "company_name",
            },
            {
                "id": "missing-password1",
                "missing_field": "password1",
            },
            {
                "id": "missing-password2",
                "missing_field": "password2",
            },
        ]

        # Act
        for case in cases:
            with self.subTest(case_id=case["id"]):
                data = dict(base_data)
                data.pop(case["missing_field"])

                form = RegistrationForm(data=data)

                # Assert
                self.assertFalse(form.is_valid())
                self.assertIn(case["missing_field"], form.errors)

    # pylint: disable=invalid-name
    def test_registration_form_all_fields_present_and_types(self):
        # Arrange
        form = RegistrationForm()

        # Act
        fields = form.fields

        # Assert
        self.assertIn("company_name", fields)
        self.assertIsInstance(fields["company_name"], forms.CharField)

        self.assertIn("email", fields)
        self.assertIsInstance(fields["email"], forms.EmailField)

        # These come from UserCreationForm
        self.assertIn("password1", fields)
        self.assertIn("password2", fields)


if __name__ == "__main__":
    unittest.main()



from django.contrib.auth.base_user import BaseUserManager


class UserManager(BaseUserManager):
    def create_user(self, email, password=None, **extra_fields):
        if not email:
            raise ValueError('Email address is mandatory')
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)

        return user

    def create_superuser(self, email, password=None, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)

        return self.create_user(email, password, **extra_fields)
